@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "license.h"
 
-DialogAbout::DialogAbout(QWidget* parent,int n_threads)
+DialogAbout::DialogAbout(QWidget* parent,int n_threads,bool separate_farm_for_enlargements)
   :QDialog(parent)
 {
   setCaption("About evolvotron");
@@ -34,31 +34,37 @@ DialogAbout::DialogAbout(QWidget* parent,int n_threads)
 
   _vbox=new QVBox(this);
 
+  QTabWidget* tabs=new QTabWidget(_vbox);
+  QVBox* vbox_info=new QVBox(tabs);
+  tabs->addTab(vbox_info,"Info");
+  QVBox* vbox_license=new QVBox(tabs);
+  tabs->addTab(vbox_license,"License");
+
   std::ostringstream about_string;
   about_string
     << "Evolvotron "
     << EVOLVOTRON_BUILD
-    << "\n"
+    << "\n\n"
     << "Using "
+    << (separate_farm_for_enlargements ? "2 pools" : "1 pool")
+    << " of "
     << n_threads
-    << " compute thread" 
-    << (n_threads>1 ? "s" : "") << "\n\n"
-    << "Author: timday@timday.com\n"
+    << " compute thread"
+    << (n_threads>1 ? "s" : "")
+    << "\n\n"
+    << "Author: timday@timday.com\n\n"
     << "Home page: http://evolvotron.sourceforge.net\n"
     << "Project page: http://sourceforge.net/projects/evolvotron\n";
 
-  _label=new QLabel(about_string.str().c_str(),_vbox);
-  _label->setAlignment(Qt::AlignHCenter|_label->alignment());
+  _label=new QLabel(about_string.str().c_str(),vbox_info);
+  _label->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter|_label->alignment());
   
-  _license=new QTextEdit(_vbox);
+  _license=new QTextEdit(vbox_license);
   _license->setReadOnly(true);
   _license->setTextFormat(PlainText);
   _license->setText((std::string("License:\n")+std::string(license_string)).c_str());
 
   _ok=new QPushButton("OK",_vbox);
-
-  //! \todo: These button settings don't seem to do anything.  Find out what's up.
-  _ok->setAutoDefault(true);
   _ok->setDefault(true);
 
   connect(
@@ -67,7 +73,8 @@ DialogAbout::DialogAbout(QWidget* parent,int n_threads)
 	  );
 }
 
-void DialogAbout::resizeEvent(QResizeEvent*)
+void DialogAbout::resizeEvent(QResizeEvent* e)
 {
+  Superclass::resizeEvent(e);
   _vbox->resize(size());
 }
