@@ -1,5 +1,5 @@
 // Source file for evolvotron
-// Copyright (C) 2002,2003,2007 Tim Day
+// Copyright (C) 2007 Tim Day
 /*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,8 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef _dialog_about_h_
 #define _dialog_about_h_
 
-#include <qtextedit.h>
-
 //! Provides an "About" dialog box.
 /*! About dialog displays author info, web addresses and license info.
  */
@@ -33,9 +31,11 @@ class DialogAbout : public QDialog
  private:
   Q_OBJECT
 
+  typedef QDialog Superclass;
+
  public:
   //! Constructor.
-  DialogAbout(QWidget* parent,int n_threads);
+  DialogAbout(QWidget* parent,int n_threads,bool separate_farm_for_enlargements);
 
   //! Destructor.
   virtual ~DialogAbout()
