@@ -23,9 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef _dialog_favourite_h_
 #define _dialog_favourite_h_
 
-#include <qcombobox.h>
-#include <qcheckbox.h>
-
 class EvolvotronMain;
 
 //! Provides a dialog for controlling which functions are available.
