@@ -25,7 +25,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "dialog_functions.h"
 
 #include <qtooltip.h>
-#include <qgroupbox.h>
 #include <qtabwidget.h>
 
 #include "evolvotron_main.h"
@@ -47,7 +46,8 @@ DialogFunctions::DialogFunctions(EvolvotronMain* parent,MutationParametersQObjec
 
   QTabWidget* tabs=new QTabWidget(_dialog_content);
   _ok=new QPushButton("OK",_dialog_content);
-    
+  _ok->setDefault(true);
+
   for (int c=-1;c<FnClassifications;c++)
     {
       QVBox*const tab_content=new QVBox(_dialog_content);
