@@ -77,20 +77,35 @@ static const char*const helptext_short=
 "<h3>General</h3>"
 "<ul>"
 "  <li>"
-"    <b>-g</b> <i>cols</i> <i>rows</i> : Set display cells in grid."
+"    <b>-a</b> : Enable autocooling by default."
 "  </li>"
 "  <li>"
-"    <b>-t</b> <i>threads</i> : Set number of compute threads."
-"  </li>"
-"  <li>"
-"    <b>-n</b> <i>niceness</i> : Set priority of compute threads relative to main (GUI) thread."
+"    <b>-E</b> : Separate thread pool for enlargements (see also -N option)."
 "  </li>"
 "  <li>"
 "    <b>-F</b> : Start in fullscreen mode."
 "  </li>"
 "  <li>"
+"    <b>-g</b> <i>cols</i> <i>rows</i> : Set display cells in grid."
+"  </li>"
+"  <li>"
+"    <b>-j</b> : Enable sample jittering."
+"  </li>"
+"  <li>"
+"    <b>-m</b> <i>multisample</i> : Final render multisampling grid (valid values are 1,2,3 or 4)."
+"  </li>"
+"  <li>"
 "    <b>-M</b> : Start with menu and status bars hidden."
 "  </li>"
+"  <li>"
+"    <b>-n</b> <i>grid niceness</i> : Set priority of compute threads relative to main (GUI) thread."
+"  </li>"
+"  <li>"
+"    <b>-N</b> <i>enlargement niceness</i> : Set priority of compute threads for enlargements relative to main (GUI) thread (only effective in conjunction with -E option)."
+"  </li>"
+"  <li>"
+"    <b>-t</b> <i>threads</i> : Set number of compute threads."
+"  </li>"
 "</ul>"
 "<h3>Animation</h3>"
 "<ul>"
@@ -98,15 +113,18 @@ static const char*const helptext_short=
 "    <b>-f</b> <i>frames</i> : Generate animations with specified number of frames."
 "  </li>"
 "  <li>"
-"    <b>-r</b> <i>frames-per-second</i> : Display animations at specified framerate."
+"    <b>-linz</b> : Vary z linearly with time instead of sinusoidally."
 "  </li>"
 "  <li>"
-"    <b>-linz</b> : Vary z linearly with time instead of sinusoidally."
+"    <b>-r</b> <i>frames-per-second</i> : Display animations at specified framerate."
 "  </li>"
 "</ul>"
 "<h3>Power User</h3>"
 "<ul>"
 "  <li>"
+"    <b>-D</b> : Debug mode where FunctionNoiseOneChannel is the dominant function.  Useful with -X for function debugging."
+"  </li>"
+"  <li>"
 "    <b>-spheremap</b> : Work with spheremaps instead of planar textures.  (NB Middle-mouse drags do not behave sensibly in this mode)."
 "  </li>"
 "  <li>"
@@ -118,9 +136,6 @@ static const char*const helptext_short=
 "  <li>"
 "    <b>-X</b> <i>function</i> : Set specified function name as \"favourite\" at top-level, with no wrapper."
 "  </li>"
-"  <li>"
-"    <b>-D</b> : Debug mode where FunctionNoiseOneChannel is the dominant function.  Useful with -X for function debugging."
-"  </li>"
 "</ul>"
 "</qt>"
 ;
@@ -141,9 +156,8 @@ DialogHelp::DialogHelp(QWidget* parent,bool full)
   _browser->setText(full ? helptext_long : helptext_short);
 
   _ok=new QPushButton("OK",_vbox);
-
-  //! \todo: These button settings don't seem to do anything.  Find out what's up.
-  _ok->setAutoDefault(true);
+  // Setting this button to default doesn't seem to have an effect (it does in other browsers).
+  // Something to do with sharing with a QTextBrowser ?
   _ok->setDefault(true);
 
   connect(
