@@ -23,21 +23,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef _dialog_mutation_parameters_h_
 #define _dialog_mutation_parameters_h_
 
-#include <qgrid.h>
-#include <qspinbox.h>
-#include <qslider.h>
-#include <qcheckbox.h>
-#include <qmainwindow.h>
-#include <qstatusbar.h>
-
 #include "mutation_parameters_qobject.h"
 
-//! Provides an "About" dialog box, manages an instance of MutationParameters.
+//! Provides an dialog box for controlling MutationParameters.
 class DialogMutationParameters : public QDialog
 {
  private:
   Q_OBJECT
 
+  typedef QDialog Superclass;
+
  protected:
   //! Scale to spinbox's integer values.
   const int _scale;
@@ -51,14 +46,32 @@ class DialogMutationParameters : public QDialog
    */
   MutationParametersQObject*const _mutation_parameters;
 
-  //! Vertical layout.
-  QVBox* _vbox;
+  //! Top level holder of all the dialog content.
+  QVBox* _dialog_content;
+
+  //! Tabs for base parameters and autocool
+  QTabWidget* _tabs;
+
+  //! Tab for base mutation parameter controls
+  QVBox* _vbox_base_mutation;
 
   //! Grid for buttons;
   QGrid* _grid_buttons;
 
-  //! Grid for parameter control spinners
-  QGrid* _grid_parameters;
+  //! Grid for base parameter control spinners
+  QGrid* _grid_base_mutation;
+
+  //! Group for autocool parameters
+  QVBox* _vbox_autocool;
+
+  //! Grid for autocool parameters
+  QGrid* _grid_autocool;
+
+  //! Label to show number of generations
+  QLabel* _label_autocool_generations;
+
+  //! Button to reheeat autocooling
+  QPushButton* _button_autocool_reheat;
 
   //@{
   //! Button for quick adjustment of MutationParameters
@@ -77,17 +90,18 @@ class DialogMutationParameters : public QDialog
   QSpinBox* _spinbox_shuffle;
   QSpinBox* _spinbox_insert;
   QSpinBox* _spinbox_substitute;
+  QSpinBox* _spinbox_autocool_halflife;
   //@}
 
+  //! Control autocooling
+  QCheckBox* _checkbox_autocool_enable;
+
   //! Button to close dialog.
   QPushButton* _ok;
 
   //! Need to pass resizes on to vbox or things just get chopped.
   virtual void resizeEvent(QResizeEvent*);
 
-  //! Put a quick message in the status bar showing the new numbers.
-  void parameters_changed_status_display();
-
   //! Reload spinboxes from _mutation_parameters.
   void setup_from_mutation_parameters();
 
@@ -102,14 +116,18 @@ class DialogMutationParameters : public QDialog
  public slots:
 
   //@{
-  //! Signalled by button (possibly status bar button).
+  //! Signalled by button.
   void reset();
   void heat();
   void cool();
   void irradiate();
   void shield();
+  void reheat();
   //@}
 
+  //! Signalled by checkbox.
+  void changed_autocool_enable(int buttonstate);
+
   //@{
   //! Signalled by spinbox.
   void changed_magnitude(int v);
@@ -118,6 +136,7 @@ class DialogMutationParameters : public QDialog
   void changed_shuffle(int v);
   void changed_insert(int v);
   void changed_substitute(int v);
+  void changed_autocool_halflife(int v);
   //@}
 
   //! Signalled by mutation parameters
