@@ -25,14 +25,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "evolvotron_main.h"
 
-#include <qmessagebox.h>
-#include <qtooltip.h>
-#include <qregexp.h>
-
 #include "args.h"
 #include "dialog_about.h"
 #include "dialog_help.h"
 #include "dialog_mutation_parameters.h"
+#include "dialog_render_parameters.h"
 #include "dialog_functions.h"
 #include "dialog_favourite.h"
 #include "function_node.h"
@@ -89,12 +86,12 @@ void EvolvotronMain::History::replacing(MutatableImageDisplay* display)
       begin_action("");
     }
   
-  const boost::shared_ptr<const MutatableImage> image=display->image();
+  const boost::shared_ptr<const MutatableImage> image_function=display->image_function();
 
-  if (image.get())
+  if (image_function.get())
     {
-      const boost::shared_ptr<const MutatableImage> saved_image(image->deepclone(image->locked()));
-      _archive.front().second.insert(std::make_pair(display,saved_image));
+      const boost::shared_ptr<const MutatableImage> saved_image_function(image_function->deepclone(image_function->locked()));
+      _archive.front().second.insert(std::make_pair(display,saved_image_function));
     }
 }
 
@@ -161,7 +158,7 @@ void EvolvotronMain::History::undo()
 	   it++
 	   )
 	{
-	  _main->restore((*it).first,(*it).second);
+	  _main->restore((*it).first,(*it).second,_archive.size()>1);
 	}
       _archive.pop_front();
     }
@@ -179,29 +176,48 @@ void EvolvotronMain::last_spawned_image(const boost::shared_ptr<const MutatableI
 /*! Constructor sets up GUI components and fires up QTimer.
   Initialises mutation parameters using time, so different every time.
  */
-EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frames,uint framerate,uint n_threads,int niceness,bool start_fullscreen,bool start_menuhidden,bool function_debug_mode)
+EvolvotronMain::EvolvotronMain
+(
+ QWidget* parent,
+ const QSize& grid_size,
+ uint frames,
+ uint framerate,
+ uint n_threads,
+ bool separate_farm_for_enlargements,
+ int niceness_grid,
+ int niceness_enlargements,
+ bool start_fullscreen,
+ bool start_menuhidden,
+ bool autocool,
+ bool jitter,
+ uint multisample_level,
+ bool function_debug_mode
+ )
   :QMainWindow(parent,0,Qt::WType_TopLevel|Qt::WDestructiveClose)
-   ,_history(new EvolvotronMain::History(this))
-   ,_mutation_parameters(time(0),function_debug_mode,this)
-   ,_statusbar_tasks(0)
-   ,_last_spawn_method(&EvolvotronMain::spawn_normal)
+  ,_history(new EvolvotronMain::History(this))
+  ,_mutation_parameters(time(0),autocool,function_debug_mode,this)
+  ,_render_parameters(jitter,multisample_level,this)
+  ,_statusbar_tasks_main(0)
+  ,_statusbar_tasks_enlargement(0)
+  ,_last_spawn_method(&EvolvotronMain::spawn_normal)
 {
   setMinimumSize(600,400);
 
-  // Need to create this first or DialogMutationParameters will cause one to be created too.
+  // Need to create this first or DialogMutationParameters might cause one to be created too.
   _statusbar=new QStatusBar(this);
   _statusbar->setSizeGripEnabled(false);
-  _statusbar->message("Ready");
 
   _statusbar_tasks_label=new QLabel("Ready",_statusbar);
   _statusbar->addWidget(_statusbar_tasks_label,0,true);
 
-  _dialog_about=new DialogAbout(this,n_threads);
+  _dialog_about=new DialogAbout(this,n_threads,separate_farm_for_enlargements);
   _dialog_help_short=new DialogHelp(this,false);
   _dialog_help_long=new DialogHelp(this,true);
 
   _dialog_mutation_parameters=new DialogMutationParameters(this,&_mutation_parameters);
 
+  _dialog_render_parameters=new DialogRenderParameters(this,&_render_parameters);
+
   _dialog_functions=new DialogFunctions(this,&_mutation_parameters);
 
   _dialog_favourite=new DialogFavourite(this);
@@ -228,12 +244,16 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
   // We want to use a checkmark on some items
   _popupmenu_settings->setCheckable(true);
 
-  _popupmenu_settings->insertItem("&Mutation parameters...",_dialog_mutation_parameters,SLOT(show()));
+  _popupmenu_settings->insertItem("Mutation &parameters...",_dialog_mutation_parameters,SLOT(show()));
   _popupmenu_settings->insertItem("&Function weightings...",_dialog_functions,SLOT(show()));
   _popupmenu_settings->insertItem("Fa&vourite function...",_dialog_favourite,SLOT(show()));
 
   _popupmenu_settings->insertSeparator();
 
+  _popupmenu_settings->insertItem("Ren&der parameters...",_dialog_render_parameters,SLOT(show()));  
+
+  _popupmenu_settings->insertSeparator();
+
   _menu_item_number_fullscreen=_popupmenu_settings->insertItem("Full&screen",this,SLOT(toggle_fullscreen()));
   _menu_item_number_hide_menu=_popupmenu_settings->insertItem("Hide &menu and statusbar",this,SLOT(toggle_hide_menu()));  
 
@@ -255,26 +275,29 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
 
   _grid=new QGrid(grid_size.width(),this);
 
-  //! \todo These might work better as QToolButton
-  _button_cool=new QPushButton("&Cool",_statusbar);
-  _button_shield=new QPushButton("&Shield",_statusbar);
-  _button_heat=new QPushButton("&Heat",_statusbar);
-  _button_irradiate=new QPushButton("&Irradiate",_statusbar);
-
-  QToolTip::add(_button_cool,"Decrease size of constant perturbations during mutation");
-  QToolTip::add(_button_heat,"Increase size of constant perturbations during mutation");
-  QToolTip::add(_button_shield,"Decrease probability of function tree structural mutations");
-  QToolTip::add(_button_irradiate,"Increase probability of function tree structural mutations");
+  _label_autocool_enable=new QLabel("",_statusbar);
+  _checkbox_autocool_enable=new QCheckBox("Autocool",_statusbar);
+  QToolTip::add(_checkbox_autocool_enable,"Autocooling gradually reduces the chance and magnitude of mutations with time.");
+  _button_autocool_reheat=new QPushButton("Reheat",_statusbar);
+  QToolTip::add(_button_autocool_reheat,"Reheat restarts the autocooling generation count, restoring the full strength of mutations.");
   
-  connect(_button_cool,     SIGNAL(clicked()),_dialog_mutation_parameters,SLOT(cool()));
-  connect(_button_heat,     SIGNAL(clicked()),_dialog_mutation_parameters,SLOT(heat()));
-  connect(_button_shield,   SIGNAL(clicked()),_dialog_mutation_parameters,SLOT(shield()));
-  connect(_button_irradiate,SIGNAL(clicked()),_dialog_mutation_parameters,SLOT(irradiate()));
+  connect(_checkbox_autocool_enable,SIGNAL(stateChanged(int)),_dialog_mutation_parameters,SLOT(changed_autocool_enable(int)));
+  connect(_button_autocool_reheat,SIGNAL(clicked()),_dialog_mutation_parameters,SLOT(reheat()));
 
-  _statusbar->addWidget(_button_cool,0,true);
-  _statusbar->addWidget(_button_shield,0,true);
-  _statusbar->addWidget(_button_heat,0,true);
-  _statusbar->addWidget(_button_irradiate,0,true);
+  _statusbar->addWidget(new QHBox(_statusbar),1,true);
+  _statusbar->addWidget(_checkbox_autocool_enable,0,true);
+  _statusbar->addWidget(_label_autocool_enable,0,true);
+  _statusbar->addWidget(_button_autocool_reheat,0,true);
+
+  connect(
+	  &_render_parameters,SIGNAL(changed()),
+	  this,SLOT(render_parameters_changed())
+	  );
+
+  connect(
+	  &_mutation_parameters,SIGNAL(changed()),
+	  this,SLOT(mutation_parameters_changed())
+	  );
   
   // We need to make sure the display grid gets all the space it can
   setCentralWidget(_grid);
@@ -286,7 +309,11 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
 	  this, SLOT(tick()) 
 	  );
 
-  _farm=std::auto_ptr<MutatableImageComputerFarm>(new MutatableImageComputerFarm(n_threads,niceness));
+  _farm[0]=std::auto_ptr<MutatableImageComputerFarm>(new MutatableImageComputerFarm(n_threads,niceness_grid));
+  if (separate_farm_for_enlargements)
+    {
+      _farm[1]=std::auto_ptr<MutatableImageComputerFarm>(new MutatableImageComputerFarm(n_threads,niceness_enlargements));
+    }
 
   //! \todo frames and framerate should be retained and modifiable from the GUI
   for (int r=0;r<grid_size.height();r++)
@@ -318,16 +345,18 @@ EvolvotronMain::~EvolvotronMain()
   std::clog << "Evolvotron shut down begun...\n";
 
   // Orphan any displays which outlived us (and clear their images)  (look out: shutdown order is Qt-determined)
+  std::clog << "(There are " << _known_displays.size() << " displays remaining)\n";
   for (std::set<MutatableImageDisplay*>::const_iterator it=_known_displays.begin();it!=_known_displays.end();it++)
     {
-      (*it)->image(boost::shared_ptr<const MutatableImage>());
+      (*it)->image_function(boost::shared_ptr<const MutatableImage>(),true);
       (*it)->main(0);
     }
 
   std::clog << "...cleared displays, deleting farm...\n";
 
-  // Shut down the compute farm
-  _farm.reset();;
+  // Shut down the compute farms
+  _farm[0].reset();
+  _farm[1].reset();
 
   std::clog << "...deleted farm, deleting history...\n";
 
@@ -354,46 +383,46 @@ void EvolvotronMain::favourite_function_unwrapped(bool v)
   _dialog_favourite->favourite_function_unwrapped(v);
 }
 
-void EvolvotronMain::spawn_normal(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display)
+void EvolvotronMain::spawn_normal(const boost::shared_ptr<const MutatableImage>& image_function,MutatableImageDisplay* display,bool one_of_many)
 {
-  boost::shared_ptr<const MutatableImage> new_image;
+  boost::shared_ptr<const MutatableImage> new_image_function;
 
   do
     {
-      new_image=image->mutated(mutation_parameters());
+      new_image_function=image_function->mutated(mutation_parameters());
     }
-  while (new_image->is_constant());
+  while (new_image_function->is_constant());
   
   history().replacing(display);
-  display->image(new_image);
+  display->image_function(new_image_function,one_of_many);
 }
 
-void EvolvotronMain::spawn_recoloured(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display)
+void EvolvotronMain::spawn_recoloured(const boost::shared_ptr<const MutatableImage>& image_function,MutatableImageDisplay* display,bool one_of_many)
 {  
-  std::auto_ptr<FunctionTop> new_root(image->top().typed_deepclone());
+  std::auto_ptr<FunctionTop> new_root(image_function->top().typed_deepclone());
   
   new_root->reset_posttransform_parameters(mutation_parameters());
   history().replacing(display);
-  boost::shared_ptr<const MutatableImage> it(new MutatableImage(new_root,image->sinusoidal_z(),image->spheremap(),false));
-  display->image(it);
+  boost::shared_ptr<const MutatableImage> it(new MutatableImage(new_root,image_function->sinusoidal_z(),image_function->spheremap(),false));
+  display->image_function(it,one_of_many);
 }
 
-void EvolvotronMain::spawn_warped(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display)
+void EvolvotronMain::spawn_warped(const boost::shared_ptr<const MutatableImage>& image_function,MutatableImageDisplay* display,bool one_of_many)
 {
-  std::auto_ptr<FunctionTop> new_root=std::auto_ptr<FunctionTop>(image->top().typed_deepclone());
+  std::auto_ptr<FunctionTop> new_root=std::auto_ptr<FunctionTop>(image_function->top().typed_deepclone());
 
   // Get the transform from whatever factory is currently set
   const Transform transform(transform_factory()(mutation_parameters().rng01()));
       
-  new_root->concatenate_pretransform_on_right(transform);  
+  new_root->concatenate_pretransform_on_right(transform);
   history().replacing(display);
-  boost::shared_ptr<const MutatableImage> it(new MutatableImage(new_root,image->sinusoidal_z(),image->spheremap(),false));
-  display->image(it);
+  boost::shared_ptr<const MutatableImage> it(new MutatableImage(new_root,image_function->sinusoidal_z(),image_function->spheremap(),false));
+  display->image_function(it,one_of_many);
 }
 
-void EvolvotronMain::restore(MutatableImageDisplay* display,const boost::shared_ptr<const MutatableImage>& image)
+void EvolvotronMain::restore(MutatableImageDisplay* display,const boost::shared_ptr<const MutatableImage>& image_function,bool one_of_many)
 {
-  if (is_known(display)) display->image(image);
+  if (is_known(display)) display->image_function(image_function,one_of_many);
 }
 
 void EvolvotronMain::set_undoable(bool v,const std::string& action_name)
@@ -418,7 +447,7 @@ void EvolvotronMain::respawn(MutatableImageDisplay* display)
 	}
       else
 	{
-	  (this->*last_spawn_method())(last_spawned_image(),display);
+	  (this->*last_spawn_method())(last_spawned_image(),display,false);
 	}
 
       history().end_action();
@@ -434,20 +463,22 @@ void EvolvotronMain::spawn_all(MutatableImageDisplay* spawning_display,SpawnMemb
 
   // Issue new images (except to locked displays and to originator)
   // This will cause them to abort any running tasks
-  const boost::shared_ptr<const MutatableImage> spawning_image(spawning_display->image());
+  const boost::shared_ptr<const MutatableImage> spawning_image_function(spawning_display->image_function());
 
-  last_spawned_image(spawning_image,method);
+  last_spawned_image(spawning_image_function,method);
   
   for (std::vector<MutatableImageDisplay*>::iterator it=displays().begin();it!=displays().end();it++)
     {
       if ((*it)!=spawning_display && !(*it)->locked())
 	{
-	  (this->*method)(spawning_image,(*it));
+	  (this->*method)(spawning_image_function,(*it),true);
 	}
     }
 
   history().end_action();
 
+  _mutation_parameters.autocool_generations_increment();
+    
   QApplication::restoreOverrideCursor();
 }
 
@@ -517,43 +548,61 @@ void EvolvotronMain::list_known(std::ostream& out) const
  */
 void EvolvotronMain::tick()
 {
-  const uint tasks=farm().tasks();
-  if (tasks!=_statusbar_tasks)
+  const uint tasks_main=_farm[0]->tasks();
+  const uint tasks_enlargement=(_farm[1].get() ? _farm[1]->tasks() : 0);
+  if (tasks_main!=_statusbar_tasks_main || tasks_enlargement!=_statusbar_tasks_enlargement)
     {
-      if (tasks==0)
-	_statusbar_tasks_label->setText("Ready");
-      else
+      std::ostringstream msg;
+      msg << "";
+
+      if (tasks_main+tasks_enlargement==0)
 	{
-	  _statusbar_tasks_label->setText(QString("%1 tasks remaining").arg(tasks));
+	  msg << "Ready";
 	}
-      _statusbar_tasks=tasks;
+      else 
+	{
+	  msg << tasks_main;
+	  if (tasks_enlargement)
+	    {
+	      msg << "+" << tasks_enlargement;
+	    }
+	  msg << " tasks remaining";
+	}
+
+      _statusbar_tasks_label->setText(msg.str().c_str());
+      _statusbar_tasks_main=tasks_main;
+      _statusbar_tasks_enlargement=tasks_enlargement;
     }
 
   boost::shared_ptr<MutatableImageComputerTask> task;
 
   // If there are aborted jobs in the todo queue 
   // shift them straight over to done queue so the compute threads don't have to worry about them.
-  farm().fasttrack_aborted();
+  _farm[0]->fasttrack_aborted();
+  if (_farm[1].get()) _farm[1]->fasttrack_aborted();
 
   QTime watchdog;
   watchdog.start();
 
-  while ((task=farm().pop_done())!=0)
+  for (int which_farm=0;which_farm<(_farm[1].get() ? 2 : 1);which_farm++)
     {
-      if (is_known(task->display()))
+      while ((task=_farm[which_farm]->pop_done())!=0)
 	{
-	  task->display()->deliver(task);
+	  if (is_known(task->display()))
+	    {
+	      task->display()->deliver(task);
+	    }
+	  else
+	    {
+	      // If we don't know who owns it we just have to trash it 
+	      // (probably a top level window which was closed with incomplete tasks).
+	      task.reset();
+	    }
+	  
+	  // Timeout in case we're being swamped by incoming tasks (maintain app responsiveness).
+	  if (watchdog.elapsed()>20)
+	    break;
 	}
-      else
-	{
-	  // If we don't know who owns it we just have to trash it 
-	  // (probably a top level window which was closed with incomplete tasks).
-	  task.reset();
-	}
-
-      // Timeout in case we're being swamped by incoming tasks (maintain app responsiveness).
-      if (watchdog.elapsed()>20)
-	break;
     }
 }    
 
@@ -655,8 +704,8 @@ void EvolvotronMain::reset(MutatableImageDisplay* display)
 
   history().replacing(display);
   //! \todo sinz and spheremap should be obtained from mutation parameters
-  const boost::shared_ptr<const MutatableImage> image(new MutatableImage(root,!Args::global().option("-linz"),Args::global().option("-spheremap"),false));
-  display->image(image);
+  const boost::shared_ptr<const MutatableImage> image_function(new MutatableImage(root,!Args::global().option("-linz"),Args::global().option("-spheremap"),false));
+  display->image_function(image_function,true);
 }
 
 void EvolvotronMain::undo()
@@ -688,9 +737,14 @@ void EvolvotronMain::reset(bool reset_mutation_parameters,bool clear_locks)
     {
       // Invoking reset on the 1st dialog actually resets the parameters 
       _dialog_mutation_parameters->reset();
-      // This one just serves to setup the dialog from the now reset parameters
+      // This one just serves to setup the function dialog from the now reset parameters
       _dialog_functions->setup_from_mutation_parameters();
     }
+  else
+    {
+      // Seems odd if we don't restart the count.
+      _mutation_parameters.autocool_generations(0);
+    }
 
   for (std::vector<MutatableImageDisplay*>::iterator it=displays().begin();it!=displays().end();it++)
     {
@@ -722,3 +776,27 @@ void EvolvotronMain::reset_cold()
   reset(true,true);
 }
 
+void EvolvotronMain::mutation_parameters_changed()
+{
+  _checkbox_autocool_enable->setChecked(_mutation_parameters.autocool_enable());
+  if (_mutation_parameters.autocool_enable())
+    {
+      _label_autocool_enable->setText(QString("Generations:")+QString::number(_mutation_parameters.autocool_generations()));
+      _label_autocool_enable->show();
+
+      if (_mutation_parameters.autocool_generations()>0) _button_autocool_reheat->show();
+      else _button_autocool_reheat->hide();
+
+    }
+  else
+    {
+      _label_autocool_enable->hide();
+      _button_autocool_reheat->hide();
+    }
+}
+
+void EvolvotronMain::render_parameters_changed()
+{
+  for (std::set<MutatableImageDisplay*>::iterator it=_known_displays.begin();it!=_known_displays.end();it++)
+    (*it)->image_function((*it)->image_function(),true);
+}
