@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef _evolvotron_main_h_
 #define _evolvotron_main_h_
 
+#include <qcheckbox.h>
 #include <qmenubar.h>
 #include <qstatusbar.h>
 #include <qmainwindow.h>
@@ -36,10 +37,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mutatable_image_display.h"
 #include "mutatable_image_computer_farm.h"
 #include "mutation_parameters_qobject.h"
+#include "render_parameters.h"
 
 class DialogAbout;
 class DialogHelp;
 class DialogMutationParameters;
+class DialogRenderParameters;
 class DialogFunctions;
 class DialogFavourite;
 
@@ -128,7 +131,7 @@ class EvolvotronMain : public QMainWindow
 
  protected:
   //! Convenience typedef for pointer to member function implementing a kind of spawn.
-  typedef void (EvolvotronMain::* SpawnMemberFn)(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display);
+  typedef void (EvolvotronMain::* SpawnMemberFn)(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display,bool one_of_many);
 
   //! Instance of History object to track activity.
   std::auto_ptr<History> _history;
@@ -139,14 +142,24 @@ class EvolvotronMain : public QMainWindow
    */
   MutationParametersQObject _mutation_parameters;
 
+  //! Instance of render parameters for the app
+  RenderParameters _render_parameters;
+
   //! Somewhere to report what's going on
   QStatusBar* _statusbar;
 
   //! Label for displaying number of tasks running (more permanent than StatusBar's message method).
   QLabel* _statusbar_tasks_label;
 
-  //! Number of tasks statusbar is reporting as active
-  uint _statusbar_tasks;
+  //! Number of main tasks the statusbar is currently reporting as active
+  /*! Cached to avoid unnecessarily regenerating message
+   */
+  uint _statusbar_tasks_main;
+
+  //! Number of enlargement tasks the statusbar is currently reporting as active
+  /*! Cached to avoid unnecessarily regenerating message
+   */
+  uint _statusbar_tasks_enlargement;
 
   //! The "About" dialog widget.
   DialogAbout* _dialog_about;
@@ -160,6 +173,9 @@ class EvolvotronMain : public QMainWindow
   //! The dialog for adjusting MutationParameters.
   DialogMutationParameters* _dialog_mutation_parameters;
 
+  //! The dialog for adjusting RenderParameters.
+  DialogRenderParameters* _dialog_render_parameters;
+
   //! Dialog for controlling which functions are in use.
   DialogFunctions* _dialog_functions;
 
@@ -190,13 +206,14 @@ class EvolvotronMain : public QMainWindow
   //! The help menu.
   QPopupMenu* _popupmenu_help;
 
-  //@{
-  //! Button for quick adjustment of MutationParameters
-  QPushButton* _button_cool;
-  QPushButton* _button_heat;
-  QPushButton* _button_shield;
-  QPushButton* _button_irradiate;
-  //@}
+  //! Select autocooling (also serves to reset the generation count).
+  QCheckBox* _checkbox_autocool_enable;
+
+  //! Report number of generations.
+  QLabel* _label_autocool_enable;
+
+  //! Button to reheat
+  QPushButton* _button_autocool_reheat;
 
   //! Grid for image display areas
   QGrid* _grid;
@@ -204,8 +221,8 @@ class EvolvotronMain : public QMainWindow
   //! Timer to drive tick() slot
   QTimer* _timer;
 
-  //! Farm of compute threads.
-  std::auto_ptr<MutatableImageComputerFarm> _farm;
+  //! Two farms of compute threads.  One for the main display, one for enlargements.
+  std::auto_ptr<MutatableImageComputerFarm> _farm[2];
 
   //! All the displays in the grid.
   std::vector<MutatableImageDisplay*> _displays;
@@ -259,9 +276,9 @@ class EvolvotronMain : public QMainWindow
 
   //@{
   //! Perform a particular type of spawn from an individiual image to an individual display.  (Locking not checked).
-  void spawn_normal(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display);
-  void spawn_recoloured(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display);
-  void spawn_warped(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display);
+  void spawn_normal(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display,bool one_of_many);
+  void spawn_recoloured(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display,bool one_of_many);
+  void spawn_warped(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display,bool one_of_many);
   //@}
 
   //! Spawn the specified display using the specified method.
@@ -269,7 +286,23 @@ class EvolvotronMain : public QMainWindow
 
  public:
   //! Constructor.
-  EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frames,uint framerate,uint n_threads,int niceness,bool start_fullscreen,bool start_menuhidden,bool function_debug_mode);
+  EvolvotronMain
+    (
+     QWidget* parent,
+     const QSize& grid_size,
+     uint frames,
+     uint framerate,
+     uint n_threads,
+     bool separate_farm_for_enlargements,
+     int niceness_grid,
+     int niceness_enlargements,
+     bool start_fullscreen,
+     bool start_menuhidden,
+     bool autocool,
+     bool jitter,
+     uint multisample_level,
+     bool function_debug_mode
+     );
 
   //! Destructor.
   ~EvolvotronMain();
@@ -295,9 +328,17 @@ class EvolvotronMain : public QMainWindow
     }
 
   //! Accessor.
-  MutatableImageComputerFarm& farm()
+  /*! NB Only const version made available publicly as modifications should be through an appropriate dialog slot.
+   */
+  const RenderParameters& render_parameters() const
     {
-      return *_farm;
+      return _render_parameters;
+    }
+
+  //! Returns which farm to use for purpose.
+  MutatableImageComputerFarm& farm(bool enlargement)
+    {
+      return *_farm[enlargement && _farm[1].get()];
     }
 
   //! Accessor.
@@ -307,7 +348,7 @@ class EvolvotronMain : public QMainWindow
     }
 
   //! Called by History when performing undo.
-  void restore(MutatableImageDisplay* display,const boost::shared_ptr<const MutatableImage>&);
+  void restore(MutatableImageDisplay* display,const boost::shared_ptr<const MutatableImage>&,bool one_of_many);
 
   //! Called by History to change undo menu status.
   void set_undoable(bool v,const std::string& name);
@@ -371,6 +412,12 @@ class EvolvotronMain : public QMainWindow
 
   //! Resets and randomizes function weightings
   void reset_randomized();
+
+  //! So we can update any exposed mutation parameters (e.g autocool enable, generation count)
+  void mutation_parameters_changed();
+
+  //! So we can re-render when render parameters change
+  void render_parameters_changed();
 };
 
 #endif
