@@ -27,15 +27,34 @@
 #include "libfunction_precompiled.h"
 
 #include <qapplication.h>
+#include <qcheckbox.h>
+#include <qcombobox.h>
 #include <qcursor.h>
 #include <qdatetime.h>
 #include <qdialog.h>
+#include <qgrid.h>
+#include <qgroupbox.h>
+#include <qhbox.h>
+#include <qhgroupbox.h>
 #include <qimage.h>
 #include <qlabel.h>
+#include <qlayout.h>
+#include <qmainwindow.h>
+#include <qmessagebox.h>
 #include <qobject.h>
 #include <qpushbutton.h>
+#include <qradiobutton.h>
+#include <qregexp.h>
+#include <qslider.h>
+#include <qspinbox.h>
+#include <qstatusbar.h>
 #include <qstring.h>
+#include <qtabwidget.h>
+#include <qtextedit.h>
+#include <qtooltip.h>
 #include <qvbox.h>
+#include <qvbuttongroup.h>
+#include <qvgroupbox.h>
 #include <qwidget.h>
 
 #endif
