@@ -31,6 +31,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "function_node_info.h"
 #include "function_top.h"
 #include "mutatable_image_display_big.h"
+#include "random.h"
 #include "transform.h"
 
 unsigned long long MutatableImage::_count=0;
@@ -87,11 +88,6 @@ boost::shared_ptr<const MutatableImage> MutatableImage::deepclone(bool lock) con
   return boost::shared_ptr<const MutatableImage>(new MutatableImage(root,sinusoidal_z(),spheremap(),lock)); 
 }
 
-const XYZ MutatableImage::operator()(const XYZ& p) const
-{
-  return top()(p);
-}
-
 const bool MutatableImage::is_constant() const
 {
   return top().is_constant();
@@ -102,19 +98,19 @@ const bool MutatableImage::ok() const
   return top().ok();
 }  
 
-const XYZ MutatableImage::sampling_coordinate(uint x,uint y,uint z,uint sx,uint sy,uint sz) const
+const XYZ MutatableImage::sampling_coordinate(real x,real y,uint z,uint sx,uint sy,uint sz) const
 {
   if (spheremap())
     {
-      const real longitude=-M_PI+2.0*M_PI*(x+0.5)/sx;
-      const real latitude=0.5*M_PI-M_PI*(y+0.5)/sy;
+      const real longitude=-M_PI+2.0*M_PI*x/sx;
+      const real latitude=0.5*M_PI-M_PI*y/sy;
       const real r=(
-		     sinusoidal_z()
-		     ?
-		     0.5+cos(M_PI*(z+0.5)/sz)
-		     :
-		     0.5+(z+0.5)/sz
-		     );
+		    sinusoidal_z()
+		    ?
+		    0.5+cos(M_PI*z/sz)
+		    :
+		    0.5+(z+0.5)/sz
+		    );
 		     
       return XYZ
 	(
@@ -127,8 +123,8 @@ const XYZ MutatableImage::sampling_coordinate(uint x,uint y,uint z,uint sx,uint
     {
       return XYZ
 	(
-	 -1.0+2.0*(x+0.5)/sx,
-	  1.0-2.0*(y+0.5)/sy,
+	 -1.0+2.0*x/sx,
+	  1.0-2.0*y/sy,
 	 (
 	  sinusoidal_z()
 	  ?
@@ -154,23 +150,51 @@ boost::shared_ptr<const MutatableImage> MutatableImage::simplified() const
   return boost::shared_ptr<const MutatableImage>(new MutatableImage(c,sinusoidal_z(),spheremap(),false));
 }
 
-void MutatableImage::get_rgb(const XYZ& p,uint c[3]) const
+const XYZ MutatableImage::get_rgb(const XYZ& p) const
 {
   // Actually calculate a pixel value from the image.
   // negexp distribution on colour-space parameters probably means the nominal range is something like -4.0 to 4.0
-  XYZ pv(top()(p));
+  const XYZ pv(top()(p));
 
-  // Scale nominal -2.0 to 2.0 range to 0-255
-  XYZ v(127.5*(0.5*pv+XYZ(1.0,1.0,1.0)));
-  
-  // Clamp out of range values 
-  v.x(clamped(v.x(),0.0,255.0));
-  v.y(clamped(v.y(),0.0,255.0));
-  v.z(clamped(v.z(),0.0,255.0));
+  // Scale a nominal -2.0 to 2.0 range to 0-255
+  return 127.5*(0.5*pv+XYZ(1.0,1.0,1.0));
+}
 
-  c[0]=(uint)floorf(v.x());
-  c[1]=(uint)floorf(v.y());
-  c[2]=(uint)floorf(v.z());
+const XYZ MutatableImage::get_rgb(uint x,uint y,uint f,uint width,uint height,uint frames,Random01* r01,uint multisample) const
+{
+  XYZ accumulated_colour(0.0,0.0,0.0);
+  for (uint sy=0;sy<multisample;sy++)
+    for (uint sx=0;sx<multisample;sx++)
+      {
+	//! \todo: Multisampling in z would be a motion blur/exposure length sort of effect (but not implemented).
+	// xyz co-ords vary over -1.0 to 1.0
+	// In the one frame case z will be 0
+	const real jx=(r01 ? (*r01)() : 0.5);
+	const real jy=(r01 ? (*r01)() : 0.5);
+	const XYZ p
+	  (
+	   sampling_coordinate
+	   (
+	    x+(sx+jx)/multisample,
+	    y+(sy+jy)/multisample,
+	    f,
+	    width,
+	    height,
+	    frames
+	    )
+	   );
+	
+	accumulated_colour+=get_rgb(p);
+      }
+
+  accumulated_colour/=(multisample*multisample);
+		  
+  // Clamp out of range values
+  accumulated_colour.x(clamped(accumulated_colour.x(),0.0,255.0));
+  accumulated_colour.y(clamped(accumulated_colour.y(),0.0,255.0));
+  accumulated_colour.z(clamped(accumulated_colour.z(),0.0,255.0));
+
+  return accumulated_colour;
 }
 
 void MutatableImage::get_stats(uint& total_nodes,uint& total_parameters,uint& depth,uint& width,real& proportion_constant) const
