@@ -37,7 +37,8 @@ MutatableImageComputer::MutatableImageComputer(MutatableImageComputerFarm* frm,i
   InstanceCounted(typeid(this).name(),false),
 #endif
   _farm(frm),
-  _niceness(niceness)
+  _niceness(niceness),
+  _r01(23)  // Seed pretty unimportant; only used for sample jitter
 {
   start();
 }
@@ -61,12 +62,13 @@ void MutatableImageComputer::run()
   std::clog << "Thread starting\n";
 
   // Lower compute thread priority slightly;
-  // computing more stuff is less important than displaying the results we've got.
+  // computing yet more stuff is less important than displaying the results we've got.
   /*! \todo: People porting to non-Linux (BSD, MacOS, Fink etc) please send
     a suitable #ifdef-able patch if you need something different here.
     Note that this code relies on Linux NPTL's non-Posix-compliant
-    thread-specific nice value.
-    \todo: Could check some error codes, but pretty harmless if it doesn't work.
+    thread-specific nice value (although without a suitable replacement
+    per-thread priority mechanism it's just as well it's that way).
+    \todo: Should check some error codes, but it's pretty harmless if it doesn't work.
   */
   const int current_priority=getpriority(PRIO_PROCESS,0);
   setpriority(PRIO_PROCESS,0,std::min(19,current_priority+_niceness));
@@ -80,32 +82,30 @@ void MutatableImageComputer::run()
 	  _task=farm()->pop_todo(*this);
 	}
       
-      // If we still don't have one might as well wait a bit before retrying
-      if (task()!=0)
+      if (task())
 	{
 	  // Careful, we could be given an already aborted task
 	  if (!task()->aborted())
 	    {
-	      const uint frames=task()->frames();
-	      const uint height=task()->size().height();
-	      const uint width=task()->size().width();
-
 	      while (!communications().kill_or_abort_or_defer() && !task()->completed())
 		{
-		  // xyz co-ords vary over -1.0 to 1.0
-		  // In the one frame case z will be 0
-		  const XYZ p(task()->image()->sampling_coordinate
-			      (
-			       task()->current_col(),task()->current_row(),task()->current_frame(),
-			       width,height,frames
-			       )
-			      );
+		  XYZ accumulated_colour=task()->image_function()->get_rgb
+		    (
+		     task()->fragment_origin().width()+task()->current_col(),
+		     task()->fragment_origin().height()+task()->current_row(),
+		     task()->current_frame(),
+		     task()->whole_image_size().width(),
+		     task()->whole_image_size().height(),
+		     task()->frames(),
+		     (task()->jittered_samples() ? &_r01 : 0),
+		     task()->multisample_grid()
+		     );
 
-		  
-		  uint c[3];
-		  task()->image()->get_rgb(p,c);
+		  const uint col0=lrint(accumulated_colour.x());
+		  const uint col1=lrint(accumulated_colour.y());
+		  const uint col2=lrint(accumulated_colour.z());
 
-		  task()->image_data()[task()->current_pixel()]=((c[0]<<16)|(c[1]<<8)|(c[2]));
+		  task()->images()[task()->current_frame()].setPixel(task()->current_col(),task()->current_row(),((col0<<16)|(col1<<8)|(col2)));
 
 		  task()->pixel_advance();
 		}
