@@ -1,5 +1,5 @@
 // Source file for evolvotron
-// Copyright (C) 2002,2003 Tim Day
+// Copyright (C) 2007 Tim Day
 /*
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -24,42 +24,84 @@
 
 #include "mutatable_image_computer_task.h"
 
-MutatableImageComputerTask::MutatableImageComputerTask(MutatableImageDisplay*const disp,const boost::shared_ptr<const MutatableImage>& img,const QSize& s,uint f,uint lev,unsigned long long int n)
+MutatableImageComputerTask::MutatableImageComputerTask
+(
+ MutatableImageDisplay*const disp,
+ const boost::shared_ptr<const MutatableImage>& fn,
+ uint pri,
+ const QSize& fo,
+ const QSize& fs,
+ const QSize& wis,
+ uint f,
+ uint lev,
+ uint frag,
+ uint nfrag,
+ bool j,
+ uint ms,
+ unsigned long long int n
+ )
   :
 #ifndef NDEBUG
   InstanceCounted(typeid(this).name(),false),
 #endif
   _aborted(false)
   ,_display(disp)
-  ,_image(img)
-  ,_size(s)
+  ,_image_function(fn)
+  ,_priority(pri)
+  ,_fragment_origin(fo)
+  ,_fragment_size(fs)
+  ,_whole_image_size(wis)
   ,_frames(f)
   ,_level(lev)
+  ,_fragment(frag)
+  ,_number_of_fragments(nfrag)
+  ,_jittered_samples(j)
+  ,_multisample_grid(ms)
   ,_current_pixel(0)
   ,_current_col(0)
   ,_current_row(0)
   ,_current_frame(0)
-  ,_image_data(new uint[s.width()*s.height()*f])
   ,_completed(false)
   ,_serial(n)
 {
-  assert(_image->ok());
+  /*
+  std::cerr 
+    << "[" 
+    << _number_of_fragments 
+    << ":" 
+    << _fragment_size.width() << "x" << _fragment_size.height() 
+    << " in " 
+    << _whole_image_size.width() << "x" << _whole_image_size.height()
+    << "]";
+  */
+  assert(_image_function->ok());
+  assert(_fragment<_number_of_fragments);
+  assert(_number_of_fragments>1 || _whole_image_size==_fragment_size);
+  assert(1<=_multisample_grid);
+}
+
+void MutatableImageComputerTask::allocate_images() const
+{
+  for (uint f=0;f<frames();f++)
+    {
+      _images.push_back(QImage(fragment_size(),32));
+    }
 }
 
 MutatableImageComputerTask::~MutatableImageComputerTask()
 {
-  assert(_image->ok());
+  assert(_image_function->ok());
 }
 
 void MutatableImageComputerTask::pixel_advance()
 {
   _current_pixel++;
   _current_col++;
-  if (_current_col==size().width())
+  if (_current_col==fragment_size().width())
     {
       _current_col=0;
       _current_row++;
-      if (_current_row==size().height())
+      if (_current_row==fragment_size().height())
 	{
 	  _current_row=0;
 	  _current_frame++;
