@@ -1,5 +1,5 @@
 // Source file for evolvotron
-// Copyright (C) 2002,2003,2007 Tim Day
+// Copyright (C) 2007 Tim Day
 /*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -47,10 +47,23 @@ class MutatableImageComputerTask
   //! The root node of the image tree to be generated.
   /*! Constness of the MutatableImage referenced is important as the instance is shared between all tasks and the original display.
    */
-  const boost::shared_ptr<const MutatableImage> _image;
+  const boost::shared_ptr<const MutatableImage> _image_function;
+
+  //! Task priority.
+  /*! Low numbers go to the head of the queue.
+    The total number of samples in the complete (non-fragmented) image is used,
+    so small low resolution images which can be quickly completed are run first.
+   */
+  const uint _priority;
+
+  //! The origin (on the display) of the image being generated.
+  const QSize _fragment_origin;
 
   //! The size of the image to be generated.
-  const QSize _size;
+  const QSize _fragment_size;
+
+  //! The full size of the image of which this is a fragment.
+  const QSize _whole_image_size;
 
   //! Number of animation frames to be rendered
   const uint _frames;
@@ -61,20 +74,36 @@ class MutatableImageComputerTask
    */
   const uint _level;
 
+  //! The fragment number, used when a rendering job is split into multiple fragments.
+  const uint _fragment;
+
+  //! The number of fragments in the rendering job
+  const uint _number_of_fragments;
+
+  //! Whether samples should be jittered.
+  const bool _jittered_samples;
+
+  //! Multisampling grid resolution e.g 4 implies a 4x4 grid
+  const uint _multisample_grid;
+
   //@{
-  //! Track pixels computed, so tasks can be restarted after defer.
+  //! Track pixels computed, so tasks can be restarted after defer.  Row and column are relative to the fragment origin.
   uint _current_pixel;
   int _current_col;
   int _current_row;
   uint _current_frame;
   //@}
 
-  //! The image data generated.
-  /*! It might have been nice to use a QImage, but every access would probably have to be qApp mutex protected.
-    (Mustn't use Qt library except from main application thread).
+  //! The image data generated for the fragments.
+  /*! This is lazily created to avoid multiple high resolution
+    images (especially with multisampling) being unnecessarily
+    concurrently allocated.
    */
-  boost::shared_array<uint> _image_data;
+  mutable std::vector<QImage> _images;
 
+  //! Lazy allocator for _images (which is mutable)
+  void allocate_images() const;
+  
   //! Set true by pixel_advance when it advances off the last frame.
   bool _completed;
 
@@ -83,7 +112,22 @@ class MutatableImageComputerTask
 
  public:
   //! Constructor.
-  MutatableImageComputerTask(MutatableImageDisplay*const disp,const boost::shared_ptr<const MutatableImage>& img,const QSize& s,uint f,uint lev,unsigned long long int n);
+  MutatableImageComputerTask
+    (
+     MutatableImageDisplay*const disp,
+     const boost::shared_ptr<const MutatableImage>& fn,
+     uint pri,
+     const QSize& fo,
+     const QSize& fs,
+     const QSize& wis,
+     uint f,
+     uint lev,
+     uint frag,
+     uint nfrag,
+     bool j,
+     uint ms,
+     unsigned long long int n
+     );
   
   //! Destructor.
   ~MutatableImageComputerTask();
@@ -107,15 +151,27 @@ class MutatableImageComputerTask
     }
 
   //! Accessor.
-  const boost::shared_ptr<const MutatableImage>& image() const
+  const boost::shared_ptr<const MutatableImage>& image_function() const
     {
-      return _image;
+      return _image_function;
     }
 
   //! Accessor.
-  const QSize& size() const
+  const QSize& fragment_origin() const
     {
-      return _size;
+      return _fragment_origin;
+    }
+
+  //! Accessor.
+  const QSize& fragment_size() const
+    {
+      return _fragment_size;
+    }
+
+  //! Accessor.
+  const QSize& whole_image_size() const
+    {
+      return _whole_image_size;
     }
 
   //! Accessor.
@@ -130,28 +186,54 @@ class MutatableImageComputerTask
       return _level;
     }
 
+  //! Accessor.
+  const uint fragment() const
+    {
+      return _fragment;
+    }
+
+  //! Accessor.
+  const uint number_of_fragments() const
+    {
+      return _number_of_fragments;
+    }
+
+  //! Accessor.
+  const bool jittered_samples() const
+    {
+      return _jittered_samples;
+    }
+
+  //! Accessor.
+  const uint multisample_grid() const
+    {
+      return _multisample_grid;
+    }
+
   //! Serial number
   const unsigned long long int serial() const
     {
       return _serial;
     }
 
-  //! Compute task priority.  Smallest images go first.
+  //! Accessor.
   const uint priority() const
     {
-      return size().width()*size().height();
+      return _priority;
+    }
+
+  //! Accessor, with lazy creation.
+  std::vector<QImage>& images()
+    {
+      if (_images.empty()) allocate_images();
+      return _images;
     }
 
   //! Accessor.
-  const boost::shared_array<uint>& image_data() const
+  const std::vector<QImage>& images() const
     {
-      return _image_data;
-    }
-
-  //! Accessor.
-  boost::shared_array<uint>& image_data()
-    {
-      return _image_data;
+      if (_images.empty()) allocate_images();
+      return _images;
     }
 
   //! Accessor.
