@@ -40,8 +40,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 /*! The constructor is passed:
     - the owning widget (probably either a QGrid or null if top-level),
     - the EvolvotronMain providing spawn and farm services, 
-    - a flag to specify whether this is a fully functional display or a restricted one (e.g no spawning for a single top pleve display),
-    - a flag to specify whether the offscreen buffer has fixed size (in which case scrollbars are used),
+    - a flag to specify whether this is a fully functional display or a restricted one (e.g no spawning for a single top level display),
+    - a flag to specify whether the offscreen buffer has fixed size (in which case scrollbars may be used),
     - and the size of the offscreen buffer (only used if fixed_size is true).
     Note that we use Qt's WDestructiveCode flag to ensure the destructor is called on close
  */
@@ -58,19 +58,15 @@ MutatableImageDisplay::MutatableImageDisplay(QWidget* parent,EvolvotronMain* mn,
    ,_timer(0)
    ,_resize_in_progress(false)
    ,_current_display_level(0)
+   ,_current_display_multisample_grid(0)
    ,_icon_serial(0LL)
    ,_properties(0)
    ,_menu(0)
    ,_menu_big(0)
    ,_serial(0LL)
 {
-  
-  // We DO want background drawn for fixed size because window could be bigger than image (not entirely satisfactory however: still flickers)
-  if (!fixed_size)
-    {
-      // This will stop the widget being cleared to grey before paintEvent is called (supposed to reduce flicker; not 100% though).
-      setBackgroundMode(Qt::NoBackground);
-    }
+  setWFlags(getWFlags()|Qt::WNoAutoErase);
+  setBackgroundMode(Qt::NoBackground);
 
   _properties=new DialogMutatableImageDisplay(this);
 
@@ -139,7 +135,7 @@ MutatableImageDisplay::MutatableImageDisplay(QWidget* parent,EvolvotronMain* mn,
   _menu->insertItem("Simplif&y function",this,SLOT(menupick_simplify()));
   _menu->insertItem("&Properties...",this,SLOT(menupick_properties()));
 
-  main()->hello(this);
+  main().hello(this);
 
   if (_fixed_size)
     {
@@ -148,7 +144,7 @@ MutatableImageDisplay::MutatableImageDisplay(QWidget* parent,EvolvotronMain* mn,
 
   for (uint f=0;f<_frames;f++)
     {
-      _offscreen_buffer.push_back(new QPixmap());
+      _offscreen_pixmaps.push_back(QPixmap());
     }
 
   _timer=new QTimer(this);
@@ -164,47 +160,47 @@ MutatableImageDisplay::MutatableImageDisplay(QWidget* parent,EvolvotronMain* mn,
  */
 MutatableImageDisplay::~MutatableImageDisplay()
 {
-  assert(!_image.get() || _image->ok());
+  assert(!_image_function.get() || _image_function->ok());
 
   // During app shutdown EvolvotronMain might have already been destroyed so only call it if it's there.
   // Don't use main() because it asserts non-null.
   if (_main)
     {
-      main()->farm().abort_for(this);
-      main()->goodbye(this);
+      farm().abort_for(this);
+      main().goodbye(this);
     }
 
-  _image.reset();
-  _offscreen_buffer.clear();
+  _image_function.reset();
+  _offscreen_pixmaps.clear();
 
-  _offscreen_image.clear();
+  _offscreen_images.clear();
 }
 
 const uint MutatableImageDisplay::simplify_constants(bool single_action)
 {
-  if (single_action) main()->history().begin_action("simplify");
+  if (single_action) main().history().begin_action("simplify");
 
   uint old_nodes;
   uint old_parameters;
   uint old_depth;
   uint old_width;
   real old_const;
-  _image->get_stats(old_nodes,old_parameters,old_depth,old_width,old_const);
+  _image_function->get_stats(old_nodes,old_parameters,old_depth,old_width,old_const);
 
-  main()->history().replacing(this);
+  main().history().replacing(this);
 
-  image(_image->simplified());
+  image_function(_image_function->simplified(),!single_action);
 
   uint new_nodes;
   uint new_parameters;
   uint new_depth;
   uint new_width;
   real new_const;
-  _image->get_stats(new_nodes,new_parameters,new_depth,new_width,new_const);
+  _image_function->get_stats(new_nodes,new_parameters,new_depth,new_width,new_const);
 
   const uint nodes_eliminated=old_nodes-new_nodes;
 
-  if (single_action) main()->history().end_action();
+  if (single_action) main().history().end_action();
 
   if (single_action)
     {
@@ -238,59 +234,119 @@ void MutatableImageDisplay::frame_advance()
 	  _animate_reverse=true;
 	}
     }
-  repaint(false);
+  repaint(false);  // Use repaint rather than update because we really do want this to happen immediately.
 }
 
-void MutatableImageDisplay::image(const boost::shared_ptr<const MutatableImage>& i)
+void MutatableImageDisplay::image_function(const boost::shared_ptr<const MutatableImage>& i,bool one_of_many)
 {
-  assert(_image.get()==0 || _image->ok());
+  assert(_image_function.get()==0 || _image_function->ok());
   assert(i.get()==0 || i->ok());
 
   // New image, so increment serial number so any old incoming stuff which somehow avoids abort is ignored.
   _serial++;
 
-  _properties->set_message(std::string("Not yet implemented"));
-
   // This might have already been done (e.g by resizeEvent), but it can't hurt to be sure.
-  main()->farm().abort_for(this);
+  farm().abort_for(this);
 
   // Careful: we could be passed our own existing (and already owned) image
   // (a trick used by resize to trigger recompute & redisplay)
-  // but if the image isn't being changed we might as well leave what's already shown
-  if (i.get()==0 || _image.get()==0 || i->serial()!=_image->serial())
+  if (i.get()==0 || _image_function.get()==0 || i->serial()!=_image_function->serial())
     {
-      _image=i;
+      _image_function=i;
 
-      // Clear any existing image data - stops old animations continuing to play 
-      for (uint f=0;f<_offscreen_buffer.size();f++)
-	_offscreen_buffer[f].fill(black);
-
-      // Queue a paint so the display will be blacked out
-      repaint();
+      // If we're part of a fullscale change then better to display back rather than something misleading
+      // but for one-offs (e.g middle mouse drag) is better not to clear.
+      if (one_of_many)
+	{
+	  // Clear any existing image data - stops old animations continuing to play 
+	  for (uint f=0;f<_offscreen_pixmaps.size();f++)
+	    _offscreen_pixmaps[f].fill(black);
+	  
+	  // Queue a redraw
+	  update();
+	}
     }
 
   // If we start recomputing again we need to accept any delivered images.
-  _current_display_level=(uint)(-1);
+  _current_display_level=static_cast<uint>(-1);
+  _current_display_multisample_grid=static_cast<uint>(-1);
+
+  // Clear up staging area... its contents are now useless
+  _offscreen_images_inbox.clear();
 
   // Update lock status displayed in menu
-  _menu->setItemChecked(_menu_item_number_lock,(_image.get() ? _image->locked() : false));
+  _menu->setItemChecked(_menu_item_number_lock,(_image_function.get() ? _image_function->locked() : false));
   
-  if (_image.get())
+  if (_image_function.get())
     {
       // Allow for displays up to 4096 pixels high or wide
       for (int level=12;level>=0;level--)
 	{
 	  const int s=(1<<level);
+	  const QSize render_size(image_size()/s);
 	  
-	  if (image_size().width()>=s && image_size().height()>=s)
+	  // Don't bother rendering anything less than 4x4 unless that's all there is
+	  if ((render_size.width()>=4 && render_size.height()>=4) || level==0)
 	    {
-	      /*! \todo Should computed frames be constant or reduced c.f spatial resolution ?  (Do full z resolution for now)
-	       */
-	      const boost::shared_ptr<const MutatableImage> task_image(_image);
-	      assert(task_image->ok());
+	      const int fragments
+		=(
+		  one_of_many
+		  ?
+		  1
+		  :
+		  std::min
+		  (
+		   2*farm().num_threads(),
+		   static_cast<uint>(render_size.height())
+		   )
+		  );
+	      
+	      std::vector<uint> multisample_grid;
+	      multisample_grid.push_back(1);
+	      if (level==0)
+		{
+		  // Only the final full resolution level gets an additional multisampling task.
+		  // For 4x4 sampling, do an initial 2x2 too.
+		  if (main().render_parameters().multisample_grid()==4) multisample_grid.push_back(2);
+		  if (main().render_parameters().multisample_grid()>1) multisample_grid.push_back(main().render_parameters().multisample_grid());
+		}
 
-	      const boost::shared_ptr<MutatableImageComputerTask> task(new MutatableImageComputerTask(this,task_image,image_size()/s,_frames,level,_serial));
-	      main()->farm().push_todo(task);
+	      for (std::vector<uint>::const_iterator multisample_it=multisample_grid.begin();multisample_it!=multisample_grid.end();multisample_it++)
+		{
+		  //! \todo Should computed animation frames be constant or reduced c.f spatial resolution ?  (Do full z resolution for now)
+		  const boost::shared_ptr<const MutatableImage> task_image(_image_function);
+		  assert(task_image->ok());
+		  
+		  // Use number of samples in unfragmented image as priority
+		  const uint task_priority=render_size.width()*render_size.height()*(*multisample_it)*(*multisample_it);
+		  
+		  int fragment_start_row=0;
+		  for (int f=0;f<fragments;f++)
+		    {
+		      const int fragment_end_row=(render_size.height()*(f+1))/fragments;
+		      const boost::shared_ptr<MutatableImageComputerTask> task
+			(
+			 new MutatableImageComputerTask
+			 (
+			  this,
+			  task_image,
+			  task_priority,
+			  QSize(0,fragment_start_row),
+			  QSize(render_size.width(),fragment_end_row-fragment_start_row),
+			  render_size,
+			  _frames,
+			  level,
+			  f,
+			  fragments,
+			  main().render_parameters().jittered_samples(),
+			  (*multisample_it),
+			  _serial
+			  )
+			 );
+		      farm().push_todo(task);
+		      fragment_start_row=fragment_end_row;
+		    }
+		}
 	    }
 	}
     }
@@ -299,90 +355,130 @@ void MutatableImageDisplay::image(const boost::shared_ptr<const MutatableImage>&
 void MutatableImageDisplay::deliver(const boost::shared_ptr<const MutatableImageComputerTask>& task)
 {
   // Ignore tasks which were aborted or which have somehow got out of order 
-  // (not impossible with multiple compute threads).
-  /*! \todo Not entirely sure the level check is sufficient in all possible situations,
-      would be best to have a serial number incremented for each recompute.
-   */
-  if (!task->aborted() && task->level()<_current_display_level && task->serial()==_serial)
+  // (entirely possible with multiple compute threads).
+  if (
+      task->aborted()
+      || task->serial()!=_serial
+      || task->level()>_current_display_level
+      || (task->level()==_current_display_level && task->multisample_grid()<=_current_display_multisample_grid)
+      )
+    return;
+
+  // Record the fragment in the inbox
+  const OffscreenImageInbox::key_type inbox_key(task->level(),task->multisample_grid());  
+  OffscreenImageInbox::mapped_type& inbox_level=_offscreen_images_inbox[inbox_key];
+  assert(inbox_level.find(task->fragment())==inbox_level.end());
+  inbox_level[task->fragment()]=task;
+  
+  if (inbox_level.size()!=task->number_of_fragments())
+    return;
+
+  // If the level is complete, we can proceed to displaying it
+
+  // Note that obsolete levels will never complete once a better one is displayed.
+  // So clear up previous levels; they're now irrelevant
+  _offscreen_images_inbox.erase
+    (
+     _offscreen_images_inbox.upper_bound(inbox_key),  // upper_bound is the NEXT key from the one we're about to display
+     _offscreen_images_inbox.end()
+     );
+  
+  const QSize render_size(task->whole_image_size());
+  
+  if (task->number_of_fragments()==1)
     {
-      _offscreen_image.clear();
-
-      _offscreen_image_data=task->image_data();
-      
-      for (uint f=0;f<task->frames();f++)
-	{
-	  _offscreen_image.push_back
-	    (
-	     new QImage
-	     (
-	      (uchar*)&(_offscreen_image_data[f*task->size().width()*task->size().height()]),
-	      task->size().width(),
-	      task->size().height(),
-	      32,
-	      0,
-	      0,
-	      QImage::LittleEndian  // Seems to be right for hi[-/R/G/B]lo packing
-	      )
-	     );
-	  
-	  //! \todo Pick a scaling mode: smooth or not (or put it under GUI control). 
-	  // Curiously, although smoothscale seems to be noticeably slower, it doesn't look any better.
-	  _offscreen_buffer[f].convertFromImage(_offscreen_image.back().scale(image_size()));
-	}
-
-      // For an icon, take the first image big enough to (hopefully) be filtered down nicely.
-      // The converter seems to auto-create an alpha mask sometimes (images with const-color areas), which is quite cool.
-      const QSize icon_size(32,32);
-      if (task->serial()!=_icon_serial && (task->level()==0 || (task->size().width()>=2*icon_size.width() && task->size().height()>=2*icon_size.height())))
-	{
-	  const QImage icon_image(_offscreen_image[_offscreen_image.size()/2].smoothScale(icon_size));
-	  
-	  _icon=std::auto_ptr<QPixmap>(new QPixmap(icon_size));
-	  _icon->convertFromImage(icon_image,QPixmap::Color);
-
-	  _icon_serial=task->serial();
-	}
-	  
-      //! Note the resolution we've displayed so out-of-order low resolution images are dropped
-      _current_display_level=task->level();
-
-      // Update what's on the screen.
-      //! \todo Any case for calling update() instead of repaint() ?  Repaint maybe feels smoother.
-      repaint();
+      // If there's only one fragment in the task, just use it
+      _offscreen_images=task->images();
     }
-  }
+  else
+    {
+      // Otherwise we need to assemble the fragments together
+      _offscreen_images.resize(0);
+      for (uint f=0;f<_frames;f++)
+	{
+	  _offscreen_images.push_back(QImage(render_size,32));
+	  
+	  for (OffscreenImageInbox::mapped_type::const_iterator it=inbox_level.begin();it!=inbox_level.end();++it)
+	    {
+	      bitBlt
+		(
+		 &_offscreen_images.back(),
+		 (*it).second->fragment_origin().width(),(*it).second->fragment_origin().height(),
+		 &(*it).second->images()[f],
+		 0,0,
+		 (*it).second->fragment_size().width(),(*it).second->fragment_size().height(),
+		 0
+		 );
+	    }
+	}
+    }
+  
+  for (uint f=0;f<_frames;f++)
+    {
+      //! \todo Pick a scaling mode: smooth or not (or put it under GUI control). 
+      // Curiously, although smoothscale seems to be noticeably slower, it doesn't look any better.
+      _offscreen_pixmaps[f].convertFromImage(_offscreen_images[f].scale(image_size()));
+    }
+  
+  //! Note the resolution we've displayed so out-of-order low resolution images are dropped
+  _current_display_level=task->level();
+  _current_display_multisample_grid=task->multisample_grid();
+  
+  // For an icon, take the first image big enough to (hopefully) be filtered down nicely.
+  // The converter seems to auto-create an alpha mask sometimes (images with const-color areas), which is quite cool.
+  const QSize icon_size(32,32);
+  if (task->serial()!=_icon_serial && (task->level()==0 || (render_size.width()>=2*icon_size.width() && render_size.height()>=2*icon_size.height())))
+    {
+      const QImage icon_image(_offscreen_images[_offscreen_images.size()/2].smoothScale(icon_size));
+      
+      _icon=std::auto_ptr<QPixmap>(new QPixmap(icon_size));
+      _icon->convertFromImage(icon_image,QPixmap::Color);
+      
+      _icon_serial=task->serial();
+    }
+
+  // Update what's on the screen.
+  update();
+}
 
 void MutatableImageDisplay::lock(bool l,bool record_in_history)
 {
   // This might be called (with l=false) with null _image during start-up reset.
-  if (_image && _image->locked()!=l)
+  if (_image_function && _image_function->locked()!=l)
     {
       if (record_in_history)
 	{
-	  main()->history().begin_action(l ? "lock" : "unlock");
-	  main()->history().replacing(this);
+	  main().history().begin_action(l ? "lock" : "unlock");
+	  main().history().replacing(this);
 	}
-      const boost::shared_ptr<const MutatableImage> new_image(_image->deepclone(l));
-      image(new_image);
+      const boost::shared_ptr<const MutatableImage> new_image_function(_image_function->deepclone(l));
+      image_function(new_image_function,false);
       if (record_in_history)
 	{
-	  main()->history().end_action();
+	  main().history().end_action();
 	}
     }
 
   _menu->setItemChecked(_menu_item_number_lock,l);
 }
 
+/*! Enlargements are implied by a non-full-functionality displays.
+ */
+MutatableImageComputerFarm& MutatableImageDisplay::farm() const
+{
+  return main().farm(!_full_functionality);
+}
+
 void MutatableImageDisplay::paintEvent(QPaintEvent*)
 {
-  // Repaint the screen from the offscreen buffer
-  // (If there have been resizes it will be black)
-  bitBlt(this,0,0,&_offscreen_buffer[_current_frame]);
+  // Repaint the screen from the offscreen pixmaps
+  // (If there have been resizes they will be black)
+  bitBlt(this,0,0,&_offscreen_pixmaps[_current_frame]);
 
   // If this is the first paint event after a resize we can start computing images for the new size.
   if (_resize_in_progress)
     {
-      image(_image);
+      image_function(_image_function,false);  // A resize should really be considered one-of-many, but because the image doesn't change we seem to be able to get away with it
       _resize_in_progress=false;
     }
 }
@@ -400,13 +496,13 @@ void MutatableImageDisplay::resizeEvent(QResizeEvent* event)
       _image_size=event->size();
       
       // Abort all current tasks because they'll be the wrong size.
-      main()->farm().abort_for(this);
+      farm().abort_for(this);
       
-      // Resize and reset our offscreen buffer (something to do while we wait)
-      for (uint f=0;f<_offscreen_buffer.size();f++)
+      // Resize and reset our offscreen pixmap (something to do while we wait)
+      for (uint f=0;f<_offscreen_pixmaps.size();f++)
 	{
-	  _offscreen_buffer[f].resize(image_size());
-	  _offscreen_buffer[f].fill(black);
+	  _offscreen_pixmaps[f].resize(image_size());  // resize destroys contents
+	  _offscreen_pixmaps[f].fill(black);           // so set it 
 	}
       
       // Flag for the next paintEvent to tell it a recompute can be started now.
@@ -423,9 +519,9 @@ void MutatableImageDisplay::mousePressEvent(QMouseEvent* event)
   else if (event->button()==MidButton)
     {
       // Take a snapshot to undo back to.
-      main()->history().begin_action("middle-button drag");
-      main()->history().replacing(this);
-      main()->history().end_action();
+      main().history().begin_action("middle-button drag");
+      main().history().replacing(this);
+      main().history().end_action();
 
       _mid_button_adjust_start_pos=event->pos();
       _mid_button_adjust_last_pos=event->pos();
@@ -560,12 +656,12 @@ void MutatableImageDisplay::mouseMoveEvent(QMouseEvent* event)
 	      std::clog << "[Pan]";
 	    }
 	  
-	  std::auto_ptr<FunctionTop> new_root(image()->top().typed_deepclone());
+	  std::auto_ptr<FunctionTop> new_root(image_function()->top().typed_deepclone());
 	  new_root->concatenate_pretransform_on_right(transform);
 
 	  // Install new image (triggers recompute).
-	  const boost::shared_ptr<const MutatableImage> new_image(new MutatableImage(new_root,image()->sinusoidal_z(),image()->spheremap(),false));
-	  image(new_image);
+	  const boost::shared_ptr<const MutatableImage> new_image_function(new MutatableImage(new_root,image_function()->sinusoidal_z(),image_function()->spheremap(),false));
+	  image_function(new_image_function,false);
 
 	  // Finally, record position of this event as last event
 	  _mid_button_adjust_last_pos=event->pos();
@@ -577,21 +673,21 @@ void MutatableImageDisplay::mouseMoveEvent(QMouseEvent* event)
  */
 void MutatableImageDisplay::menupick_respawn()
 {
-  main()->respawn(this);
+  main().respawn(this);
 }
 
 /*! This slot is called by selecting the "Spawn" context menu item, or by clicking the image
  */
 void MutatableImageDisplay::menupick_spawn()
 {
-  main()->spawn_normal(this);
+  main().spawn_normal(this);
 }
 
 /*! This slot is called by selecting the "Spawn Recoloured" context menu item
  */
 void MutatableImageDisplay::menupick_spawn_recoloured()
 {
-  main()->spawn_recoloured(this);
+  main().spawn_recoloured(this);
 }
 
 /*! This slot is called by selecting the "Spawn Warped/Random" context menu item
@@ -600,56 +696,56 @@ void MutatableImageDisplay::menupick_spawn_warped_random()
 {
   TransformFactoryRandomWarpXY transform_factory;
   
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_zoom_in()
 {
   TransformFactoryRandomScaleXY transform_factory(-2.0,0.0);
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_zoom_out()
 {
   TransformFactoryRandomScaleXY transform_factory(0.0,2.0);
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_rotate()
 {
   TransformFactoryRandomRotateZ transform_factory;
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_pan_xy()
 {
   TransformFactoryRandomTranslateXYZ transform_factory(XYZ(0.0,0.0,0.0),XYZ(1.0,1.0,0.0));
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_pan_x()
 {
   TransformFactoryRandomTranslateXYZ transform_factory(XYZ(0.0,0.0,0.0),XYZ(1.0,0.0,0.0));
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_pan_y()
 {
   TransformFactoryRandomTranslateXYZ transform_factory(XYZ(0.0,0.0,0.0),XYZ(0.0,1.0,0.0));
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 void MutatableImageDisplay::menupick_spawn_warped_pan_z()
 {
   TransformFactoryRandomTranslateXYZ transform_factory(XYZ(0.0,0.0,0.0),XYZ(0.0,0.0,1.0));
 
-  main()->spawn_warped(this,transform_factory);
+  main().spawn_warped(this,transform_factory);
 }
 
 /*! This slot is called by selecting the "Lock" context menu item.
@@ -657,7 +753,7 @@ void MutatableImageDisplay::menupick_spawn_warped_pan_z()
  */
 void MutatableImageDisplay::menupick_lock()
 {
-  lock(!_image->locked(),true);
+  lock(!_image_function->locked(),true);
 }
 
 void MutatableImageDisplay::menupick_simplify()
@@ -674,7 +770,7 @@ void MutatableImageDisplay::menupick_save_image()
 
   std::clog << "Save requested...\n";
 
-  if (_current_display_level!=0)
+  if (_current_display_level!=0 || _current_display_multisample_grid!=main().render_parameters().multisample_grid())
     {
       QMessageBox::information(this,"Evolvotron","The selected image has not yet been generated at maximum resolution.\nPlease try again later.");
     }
@@ -720,9 +816,9 @@ void MutatableImageDisplay::menupick_save_image()
 		    {
 		      QPNGImagePacker packer(mng_file.get(),32,0);
 		      
-		      for (uint f=0;f<_offscreen_image.size();f++)
+		      for (uint f=0;f<_offscreen_images.size();f++)
 			{
-			  if (!packer.packImage(_offscreen_image[f]))
+			  if (!packer.packImage(_offscreen_images[f]))
 			    {
 			      QMessageBox::critical(this,"Evolvotron","Failed while writing file "+save_filename+"\nFile will be removed");
 			      mng_file->close();
@@ -748,11 +844,11 @@ void MutatableImageDisplay::menupick_save_image()
 		}
 	      else
 		{
-		  for (uint f=0;f<_offscreen_image.size();f++)
+		  for (uint f=0;f<_offscreen_images.size();f++)
 		    {
 		      QString actual_save_filename(save_filename);
 		      
-		      if (_offscreen_image.size()>1)
+		      if (_offscreen_images.size()>1)
 			{
 			  QString frame_component;
 			  frame_component.sprintf(".f%06d",f);
@@ -767,10 +863,10 @@ void MutatableImageDisplay::menupick_save_image()
 			    }
 			}
 		      
-		      if (!_offscreen_image[f].save(actual_save_filename.local8Bit(),save_format))
+		      if (!_offscreen_images[f].save(actual_save_filename.local8Bit(),save_format))
 			{
 			  QMessageBox::critical(this,"Evolvotron","Failed to write file "+actual_save_filename);
-			  if (f<_offscreen_image.size()-1)
+			  if (f<_offscreen_images.size()-1)
 			    {
 			      QMessageBox::critical(this,"Evolvotron","Not attempting to save remaining images in animation");
 			    }
@@ -792,7 +888,7 @@ void MutatableImageDisplay::menupick_save_function()
   if (!save_filename.isEmpty())
     {
       std::ofstream file(save_filename.local8Bit());
-      _image->save_function(file);
+      _image_function->save_function(file);
       file.flush();
       if (!file)
 	{
@@ -808,8 +904,8 @@ void MutatableImageDisplay::menupick_load_function()
     {
       std::ifstream file(load_filename.local8Bit());
       std::string report;
-      boost::shared_ptr<const MutatableImage> new_image(MutatableImage::load_function(_main->mutation_parameters().function_registry(),file,report));
-      if (new_image.get()==0)
+      boost::shared_ptr<const MutatableImage> new_image_function(MutatableImage::load_function(_main->mutation_parameters().function_registry(),file,report));
+      if (new_image_function.get()==0)
 	{
 	  QMessageBox::critical(this,"Evolvotron",("Function not loaded due to errors:\n"+report).c_str());
 	}
@@ -820,10 +916,10 @@ void MutatableImageDisplay::menupick_load_function()
 	      QMessageBox::warning(this,"Evolvotron",("Function loaded with warnings:\n"+report).c_str(),QMessageBox::Ok,QMessageBox::NoButton);
 	    }
 	  
-	  main()->history().begin_action("load");
-	  main()->history().replacing(this);
-	  main()->history().end_action();
-	  image(new_image);
+	  main().history().begin_action("load");
+	  main().history().replacing(this);
+	  main().history().end_action();
+	  image_function(new_image_function,false);
 	}
     }
 }
@@ -891,18 +987,20 @@ void MutatableImageDisplay::menupick_properties()
   uint width;
   real proportion_constant;
 
-  image()->get_stats(total_nodes,total_parameters,depth,width,proportion_constant);
+  image_function()->get_stats(total_nodes,total_parameters,depth,width,proportion_constant);
 
   std::stringstream msg;
-  msg << "Function nodes  \t" << total_nodes << "\n";
-  msg << "Parameters      \t" << total_parameters << "\n";
-  msg << "Maximum depth   \t" << depth << "\n";
-  msg << "Width           \t" << width << "\n";
-  msg << "Constant        \t" << 100.0*proportion_constant << "%\n";
+  msg << " " << total_nodes      << "\t function nodes\n";
+  msg << " " << total_parameters << "\t parameters\n";
+  msg << " " << depth            << "\t maximum depth\n";
+  msg << " " << width            << "\t width\n";
+  msg << " " << std::setprecision(3) << 100.0*proportion_constant << "%\t constant\n";
 
+  std::stringstream xml;
+  image_function()->save_function(xml);
+
+  _properties->set_content(msg.str(),xml.str());
   if (_icon.get()) _properties->setIcon(*_icon);
-
-  _properties->set_message(msg.str());
   _properties->exec();
 }
 
@@ -911,7 +1009,7 @@ void MutatableImageDisplay::menupick_properties()
 */
 void MutatableImageDisplay::spawn_big(bool scrollable,const QSize& sz)
 {
-  MutatableImageDisplayBig*const top_level_widget=new MutatableImageDisplayBig(0,main());
+  MutatableImageDisplayBig*const top_level_widget=new MutatableImageDisplayBig(0,&main());
   if (_icon.get()) top_level_widget->setIcon(*_icon);
 
   MutatableImageDisplay* display=0;
@@ -919,21 +1017,21 @@ void MutatableImageDisplay::spawn_big(bool scrollable,const QSize& sz)
   if (scrollable)
     {
       QScrollView*const scrollview=new QScrollView(top_level_widget,0,Qt::WDestructiveClose);
-      display=new MutatableImageDisplay(scrollview->viewport(),main(),false,true,sz,_frames,_framerate);
+      display=new MutatableImageDisplay(scrollview->viewport(),&main(),false,true,sz,_frames,_framerate);
       scrollview->addChild(display);
       top_level_widget->hold(scrollview);
     }
   else
     {
-      display=new MutatableImageDisplay(top_level_widget,main(),false,false,QSize(0,0),_frames,_framerate);
+      display=new MutatableImageDisplay(top_level_widget,&main(),false,false,QSize(0,0),_frames,_framerate);
       top_level_widget->hold(display);
     }
 
   top_level_widget->show();
 
   //Propagate full screen mode 
-  if (main()->isFullScreen()) top_level_widget->showFullScreen();
+  if (main().isFullScreen()) top_level_widget->showFullScreen();
 
   // Fire up image calculation
-  display->image(_image);
+  display->image_function(_image_function,false);
 }
