@@ -46,10 +46,10 @@ class MutatableImageDisplay : public QWidget
   EvolvotronMain* _main;
 
   //! Flag for whether context menu should display all options.
-  bool _full_functionality;
+  const bool _full_functionality;
 
   //! Flag for whether the offscreen buffer has fixed size
-  bool _fixed_size;
+  const bool _fixed_size;
 
   //! Size of offscreen buffer
   QSize _image_size;
@@ -75,10 +75,13 @@ class MutatableImageDisplay : public QWidget
   bool _resize_in_progress;
 
   //! The resolution level currently displaying (0=1-for-1 pixels, 1=half resolution etc).
-  /*! Needed to handle possible (but unlikely?) out of order task returns from multiple compute threads.
+  /*! Needed to handle possible out of order task returns from multiple compute threads.
    */
   uint _current_display_level;
 
+  //! Similar to _current_display_level, but for tracking multisample grids within a resolution level.
+  uint _current_display_multisample_grid;
+
   //! An image suitable for setting as an icon.
   std::auto_ptr<QPixmap> _icon;
 
@@ -86,19 +89,26 @@ class MutatableImageDisplay : public QWidget
   unsigned long long int _icon_serial;
 
   //! Offscreen image buffer.
-  boost::ptr_vector<QPixmap> _offscreen_buffer;
+  std::vector<QPixmap> _offscreen_pixmaps;
 
   //! Offscreen image buffer in sensible image format (used for save, as pixmap is in display format which might be less bits).
-  boost::ptr_vector<QImage> _offscreen_image;
+  std::vector<QImage> _offscreen_images;
 
-  //! Offscreen image data for _offscreen_image.  This must remain alive longer than the QImage.
-  boost::shared_array<uint> _offscreen_image_data;
+  //! Type for staging area for incoming fragments.
+  /*! Key is level and multisampling, mapped type is also itself a map from fragment number to tasks.
+   */
+  typedef std::map<std::pair<uint,uint>,std::map<uint,boost::shared_ptr<const MutatableImageComputerTask> > > OffscreenImageInbox;
+
+  //! Staging area for incoming fragments.
+  /*! Fragments are accumulated for each (level,multisample) key, and completed levels passed on for display
+   */
+  OffscreenImageInbox _offscreen_images_inbox;
 
   //! The image function being displayed (its root node).
   /*! The held image is const because references to it could be held by history archive, compute tasks etc,
     so it should be completely replaced rather than manipulated.
    */
-  boost::shared_ptr<const MutatableImage> _image;
+  boost::shared_ptr<const MutatableImage> _image_function;
 
   //! Properties dialog.
   DialogMutatableImageDisplay* _properties;
@@ -134,24 +144,24 @@ class MutatableImageDisplay : public QWidget
   virtual ~MutatableImageDisplay();
 
   //! Accessor.
-  const boost::shared_ptr<const MutatableImage>& image()
+  const boost::shared_ptr<const MutatableImage>& image_function()
     {
-      return _image;
+      return _image_function;
     }
 
   //! Accessor.
   const bool locked() const
     {
-      return (_image.get()!=0 ? _image->locked() : false);
+      return (_image_function.get()!=0 ? _image_function->locked() : false);
     }
 
   //! Accessor.
-  EvolvotronMain*const main() const
+  EvolvotronMain& main() const
     {
       assert(_main!=0);
-      return _main;
+      return *_main;
     }
-
+  
   //! Accessor.
   void main(EvolvotronMain* m)
     {
@@ -165,7 +175,10 @@ class MutatableImageDisplay : public QWidget
     }
 
   //! Load a new image (clears up old image, starts new compute tasks).
-  void image(const boost::shared_ptr<const MutatableImage>& image);
+  /*! When the one_of_many parameter is true, it implies many other images are also being updated
+    (affects fragmentation strategy for multithreading).
+   */
+  void image_function(const boost::shared_ptr<const MutatableImage>& image_fn,bool one_of_many);
 
   //! Evolvotron main calls this with completed (but possibly aborted) tasks.
   void deliver(const boost::shared_ptr<const MutatableImageComputerTask>& task);
@@ -174,6 +187,10 @@ class MutatableImageDisplay : public QWidget
   void lock(bool l,bool record_in_history);
 
  protected:
+
+  //! Which farm this display should use.
+  MutatableImageComputerFarm& farm() const;
+
   //! Usual handler for repaint events.
   virtual void paintEvent(QPaintEvent* event);
 
