@@ -1,5 +1,5 @@
 // Source file for evolvotron
-// Copyright (C) 2002,2003,2007 Tim Day
+// Copyright (C) 2007 Tim Day
 /*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,9 +24,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "mutation_parameters_qobject.h"
 
-MutationParametersQObject::MutationParametersQObject(uint seed,bool debug_mode,QObject* parent)
+MutationParametersQObject::MutationParametersQObject(uint seed,bool autocool,bool debug_mode,QObject* parent)
   :QObject(parent)
-  ,MutationParameters(seed,debug_mode)
+  ,MutationParameters(seed,autocool,debug_mode)
 {}
 
 MutationParametersQObject::~MutationParametersQObject()
